from os.path import join
import sys
import os
import random
import shutil

try:
    import configparser
except ImportError:
    # Python 2.x fallback
    import ConfigParser as configparser
from time import time

# COMPSs/PyCOMPSs imports
from pycompss.api.task import task
from pycompss.api.binary import binary
from pycompss.api.constraint import constraint
from pycompss.api.mpi import mpi
from pycompss.api.api import compss_wait_on
from pycompss.api.api import compss_barrier
from pycompss.api.api import TaskGroup
from pycompss.api.api import compss_barrier_group
from pycompss.api.parameter import *
from pycompss.api.on_failure import on_failure

# project imports
from esm_ensemble_init import *
#from esm_pruning import *

#deprecated
# @task()
# def test_pruning():
#     itimestep = 1
#     t = threading.Timer(5.0, test_pruning)
#     t.start()
#     print("################################################### lauching pruning " + str(itimestep) +  "###################################################")
#

#deprecated
# @task(log_pruning={Type:FILE_OUT, StdIOStream:STDOUT})
# def esm_pruning(exp_id, log_pruning):
#    # 1 - connect to hecuba and obtain values
#    print("################################################### lauching pruning agent ###################################################")
#    # 2 - for each simulation to be stopped, remove it from the task group
#    s = sched.scheduler(time.time, time.sleep)
#    s.enter(10, 1, get_members_state, [s,exp_id])
#    s.run()
#    pass

# TODO: check why putting the working dir in the task in the task doesnt work and crashes
#@mpi(binary="${FESOM_WORKINGDIR}/${FESOM_EXE}", runner="mpirun", processes="${FESOM_CORES}", working_dir="${FESOM_WORKINGDIR}", fail_by_exit_value=True)
#@on_failure(management ='CANCEL_SUCCESSORS')
#@mpi(binary=os.environ['FESOM_WORKINGDIR'] + "/${FESOM_EXE}", runner="mpirun", processes="${FESOM_CORES}", working_dir=os.environ['FESOM_WORKINGDIR'], fail_by_exit_value=True)
#@task(log_file={Type:FILE_OUT, StdIOStream:STDOUT}, working_dir={Type:IN})
#def esm_simulation(log_file, working_dir):
#   pass

#@mpi(binary="${FESOM_WORKINGDIR}/${FESOM_EXE}", runner="mpirun", processes="${FESOM_CORES}", working_dir=os.environ['FESOM_WORKINGDIR'], fail_by_exit_value=True)
@on_failure(management='CANCEL_SUCCESSORS')
@mpi(binary=os.environ['FESOM_WORKINGDIR'] + "/${FESOM_EXE}", runner="mpirun", processes="${FESOM_CORES}", working_dir=os.environ['FESOM_WORKINGDIR'], fail_by_exit_value=True)
@task(log_file={Type:FILE_OUT, StdIOStream:STDOUT})
def esm_simulation(log_file):
   pass



# function to run ESM ensemble, each one should have different start dates or initial conditions/ forcings
# for making sense
def esm_ensemble(exp_id):

   # 1 - Creation of group
   with TaskGroup(str(exp_id), False):
      #3 - Launch the ensemble members
      # TODO: add the runtime path and output path of each on
      top_working_dir = exp_settings['common']['top_working_dir']
      sdates_list = (exp_settings['common']['ensemble_start_dates']).split()
      for sdate in sdates_list:
         working_dir = top_working_dir + "/" + sdate
         os.environ['FESOM_WORKINGDIR'] = working_dir
         log = working_dir + "/" + "fesom2_" + str(exp_id) + "_" + str(sdate) + ".out"
         print("################################################### launching simulation in " + working_dir + "###################################################")
         #esm_simulation(log,working_dir)
         esm_simulation(log)

   compss_barrier_group(str(exp_id))
   # create flag file to stop the parallel process
   run_finished_flag =  str(exp_id) + ".finished"
   open(top_working_dir + "/" + run_finished_flag , 'a').close()

if __name__ == "__main__":
   print("Running FESOM2 -Pycompss")
   # 0 - create a ramdon nr for the experiment id
   exp_id = random.randint(100000, 999999)
   # prepare folder structure/configuration
   exp_settings = compss_wait_on(esm_ensemble_init(exp_id))
   # exp_settings =  compss_wait_on(exp_settings)
   print("##################################### Initialization completed ####################################")
   # run the experiment
   esm_ensemble(exp_id)


